@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 2006-2019 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright © 2006-2021 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -224,7 +224,11 @@ extern void check_gl_error (const char *type);
     // attrs[i++] = NSOpenGLPFANoRecovery;
   }
 
+# pragma clang diagnostic push   // "NSOpenGLPFAWindow deprecated in 10.9"
+# pragma clang diagnostic ignored "-Wdeprecated"
   attrs[i++] = NSOpenGLPFAWindow;
+# pragma clang diagnostic pop
+
 # ifdef JWXYZ_GL
   attrs[i++] = NSOpenGLPFAPixelBuffer;
 # endif
