@@ -86,6 +86,7 @@ struct state {
   Bitmap *theBitmap; /* ginormo bitmap for drawing */
 
   enum { BC_SCROLL, BC_GRID, BC_CLOCK12, BC_CLOCK24 } mode;
+  int step;
 
   int grid_w;
   int grid_h;
@@ -104,6 +105,7 @@ static const char *words[] =
   "affluenza",
   "alertness",
   "Algeria",
+  "all your base",
   "antifa",
   "anxiety",
   "aorta",
@@ -119,16 +121,20 @@ static const char *words[] =
   "bellicosity",
   "bells",
   "belly",
+  "bird flu",
+  "Bitcorn",
   "bliss",
   "bogosity",
   "boobies",
   "boobs",
   "booty",
   "bread",
+  "bribe",
   "brogrammers",
   "bubba",
   "burrito",
   "California",
+  "cancer",
   "capybara",
   "cardinality",
   "caribou",
@@ -143,6 +149,8 @@ static const char *words[] =
   "contrition",
   "cop",
   "corpse",
+  "coronavirus",
+  "covid-19",
   "cowboy",
   "crabapple",
   "craziness",
@@ -162,14 +170,17 @@ static const char *words[] =
   "DNA Lounge",
   "doberman",
   "DOOM",
+  "doom loop",
   "dot com",
   "dreams",
   "drugs",
+  "Dunning-Krugerrands",
   "easy",
   "ebony",
   "election",
   "eloquence",
   "emergency",
+  "emolument",
   "eureka",
   "excommunication",
   "fat",
@@ -191,6 +202,8 @@ static const char *words[] =
   "goggles",
   "goobers",
   "gorilla",
+  "guillotine",
+  "H5N1",
   "halibut",
   "handmaid",
   "happiness",
@@ -201,6 +214,7 @@ static const char *words[] =
   "heroin",
   "heroine",
   "hope",
+  "horse paste",
   "hysteria",
   "icepick",
   "identity",
@@ -250,10 +264,14 @@ static const char *words[] =
   "nationalism",
   "nature",
   "neuron",
+  "NFTs",
   "noise",
   "nomenclature",
+  "NULL",
+  "null island",
   "nutria",
   "OBEY",
+  "ouroboros",
   "ocelot",
   "offspring",
   "overseer",
@@ -297,6 +315,8 @@ static const char *words[] =
   "roadrunner",
   "rootkit",
   "rule",
+  "SARS",
+  "SARS-CoV-2",
   "savor",
   "scab",
   "scalar",
@@ -328,6 +348,7 @@ static const char *words[] =
   "television",
   "tenant",
   "tendril",
+  "teratoma",
   "terror",
   "terrorism",
   "terrorist",
@@ -336,10 +357,12 @@ static const char *words[] =
   "the unknown",
   "toast",
   "topography",
+  "tribble",
   "truism",
   "truthiness",
   "turgid",
   "twits",
+  "undef",
   "underbrush",
   "underling",
   "unguent",
@@ -379,6 +402,7 @@ static const char *words[] =
   "yellow",
   "yesterday",
   "your nose",
+  "Y2038",
   "Zanzibar",
   "zeal",
   "zebra",
@@ -1536,6 +1560,11 @@ static void setup (struct state *st)
     st->windowWidth = xgwa.width;
     st->windowHeight = xgwa.height;
 
+    st->step = get_integer_resource (st->dpy, "step", "Step");
+    if (st->step < 1) st->step = 1;
+    if (st->windowWidth > 2560 || st->windowHeight > 2560)
+      st->step *= 3;  /* Retina displays */
+
     gcv.background = get_pixel_resource (st->dpy, xgwa.colormap,
                                          "background", "Background");
     gcv.foreground = get_pixel_resource (st->dpy, xgwa.colormap,
@@ -1627,7 +1656,7 @@ static void scrollModel (struct state *st)
     for (i = 0; i < st->barcode_count; i++) 
     {
 	Barcode *b = &st->barcodes[i];
-	b->x--;
+	b->x -= st->step;
 	if ((b->x + BARCODE_WIDTH * b->mag) < 0) 
 	{
 	    /* fell off the edge */
@@ -1949,8 +1978,9 @@ barcode_free (Display *dpy, Window window, void *closure)
 static const char *barcode_defaults [] = {
     ".background:	black",
     ".foreground:	green",
-    ".lowrez:           true",
+/*  ".lowrez:           true", */
     "*fpsSolid: 	true",
+    "*step:	 	1",
     "*delay:		10000",
     "*mode:		scroll",
     0
@@ -1960,6 +1990,7 @@ static XrmOptionDescRec barcode_options [] = {
   { "-delay",            ".delay",          XrmoptionSepArg, 0 },
   { "-mode",             ".mode",           XrmoptionSepArg, 0 },
   { "-scroll",           ".mode",           XrmoptionNoArg, "scroll"  },
+  { "-step",             ".step",           XrmoptionNoArg, "step"    },
   { "-grid",             ".mode",           XrmoptionNoArg, "grid"    },
   { "-clock",            ".mode",           XrmoptionNoArg, "clock"   },
   { "-clock12",          ".mode",           XrmoptionNoArg, "clock12" },
