@@ -16,7 +16,6 @@
 # include "xdbe.h"
 #endif /* HAVE_DOUBLE_BUFFER_EXTENSION */
 
-#define countof(x) (sizeof(x)/sizeof(*(x)))
 #define ABS(x) ((x)<0?-(x):(x))
 #define MAX(x,y) ((x)<(y)?(y):(x))
 #define MIN(x,y) ((x)>(y)?(y):(x))
@@ -835,7 +834,8 @@ compass_init (Display *dpy, Window window)
 
   {
     int max = 600;
-    if (st->xgwa.width > 2560) max *= 2;  /* Retina displays */
+    if (st->xgwa.width > 2560 || st->xgwa.height > 2560)
+      max *= 2;  /* Retina displays */
     if (st->size2 > max) st->size2 = max;
   }
 
