@@ -19,25 +19,12 @@
 #define DEFAULTS	    "*delay:   20000   \n" \
 			    "*showFPS: False   \n"
 
-# define refresh_antinspect 0
 # define release_antinspect 0
 #include "xlockmore.h"
 #else
 #include "xlock.h"
 #endif
 
-#ifdef HAVE_JWXYZ
-# include "jwxyz.h"
-#else
-# include <X11/Xlib.h>
-# include <GL/gl.h>
-# include <GL/glu.h>
-#endif
-
-#ifdef HAVE_JWZGLES
-# include "jwzgles.h"
-#endif /* HAVE_JWZGLES */
-
 #include "sphere.h"
 #include "gltrackball.h"
 
@@ -549,10 +536,17 @@ static Bool draw_antinspect_strip(ModeInfo * mi)
 ENTRYPOINT void reshape_antinspect(ModeInfo * mi, int width, int height) 
 {
   double h = (GLfloat) height / (GLfloat) width;  
+  int y = 0;
   antinspectstruct *mp = &antinspect[MI_SCREEN(mi)];
   mp->linewidth = (width / 512) + 1;
 
-  glViewport(0, 0, mp->WindW = (GLint) width, mp->WindH = (GLint) height);
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport(0, y, mp->WindW = (GLint) width, mp->WindH = (GLint) height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
 
@@ -605,7 +599,7 @@ ENTRYPOINT void init_antinspect(ModeInfo * mi)
 {
   antinspectstruct *mp;
   
-  MI_INIT(mi, antinspect, NULL);
+  MI_INIT(mi, antinspect);
   mp = &antinspect[MI_SCREEN(mi)];
   mp->step = NRAND(90);
   mp->ant_position = NRAND(90);
@@ -638,7 +632,7 @@ ENTRYPOINT void draw_antinspect(ModeInfo * mi)
   if(!mp->glx_context)
 	return;
   
-  glXMakeCurrent(display, window, *(mp->glx_context));
+  glXMakeCurrent(display, window, *mp->glx_context);
   
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -650,15 +644,12 @@ ENTRYPOINT void draw_antinspect(ModeInfo * mi)
      the antbubble */
   glTranslatef(0.0, 0.0, -10.0);
 
-# ifdef HAVE_MOBILE	/* Keep it the same relative size when rotated. */
   {
-    GLfloat h = MI_HEIGHT(mi) / (GLfloat) MI_WIDTH(mi);
-    int o = (int) current_device_rotation();
-    if (o != 0 && o != 180 && o != -180)
-      glScalef (1/h, 1/h, 1/h);
-    glRotatef(o, 0, 0, 1);
+    GLfloat s = (MI_WIDTH(mi) < MI_HEIGHT(mi)
+                 ? (MI_WIDTH(mi) / (GLfloat) MI_HEIGHT(mi))
+                 : 1);
+    glScalef (s, s, s);
   }
-# endif
 
   gltrackball_rotate(mp->trackball);
   glRotatef((15.0/2.0 + 15.0*sin(mp->ant_step/100.0)), 1.0, 0.0, 0.0);
@@ -688,10 +679,17 @@ ENTRYPOINT void change_antinspect(ModeInfo * mi)
   if (!mp->glx_context)
 	return;
   
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(mp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *mp->glx_context);
   pinit();
 }
 #endif /* !STANDALONE */
 
+ENTRYPOINT void free_antinspect(ModeInfo * mi) 
+{
+  antinspectstruct *mp = &antinspect[MI_SCREEN(mi)];
+  if (!mp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *mp->glx_context);
+  gltrackball_free (mp->trackball);
+}
 
 XSCREENSAVER_MODULE ("AntInspect", antinspect)
