@@ -9,20 +9,6 @@
 #ifndef __BUILD_LWO_H__
 #define __BUILD_LWO_H__
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif /* HAVE_CONFIG_H */
-
-#ifdef STANDALONE
-# ifndef HAVE_COCOA
-#  include <GL/gl.h>
-# endif
-#endif
-
-#ifdef HAVE_JWZGLES
-# include "jwzgles.h"
-#endif /* HAVE_JWZGLES */
-
 struct lwo {
 	int             num_pnts;
 	const GLfloat  *pnts;
