@@ -22,16 +22,11 @@
  * which can be obtained from http://www.linas.org/gle/index.html
   */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef STANDALONE
 #define	DEFAULTS	"*delay:	 20000	\n" \
 					"*showFPS:	 False	\n" \
 					"*wireframe: False   \n"
 
-# define free_extrusion 0
 # define release_extrusion 0
 # include "xlockmore.h"				/* from the xscreensaver distribution */
 #else /* !STANDALONE */
@@ -40,17 +35,6 @@
 
 #ifdef USE_GL /* whole file */
 
-#ifdef HAVE_XMU
-# ifndef VMS
-#  include <X11/Xmu/Drawing.h>
-#else  /* VMS */
-#  include <Xmu/Drawing.h>
-# endif /* VMS */
-#endif
-
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
-
 #include "ximage-loader.h"
 #include "rotator.h"
 #include "gltrackball.h"
@@ -314,7 +298,7 @@ draw_extrusion(ModeInfo * mi)
   if (!gp->glx_context)
 	return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(gp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *gp->glx_context);
 
   glPushMatrix();
 
@@ -551,6 +535,17 @@ init_extrusion (ModeInfo * mi)
 
 }
 
+
+ENTRYPOINT void
+free_extrusion (ModeInfo * mi)
+{
+  extrusionstruct *gp = &Extrusion[MI_SCREEN(mi)];
+  if (!gp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *gp->glx_context);
+  if (gp->trackball) gltrackball_free (gp->trackball);
+  if (gp->rot) free_rotator (gp->rot);
+}
+
 XSCREENSAVER_MODULE ("Extrusion", extrusion)
 
 #endif  /* USE_GL */
