@@ -748,7 +748,8 @@ static void inter_init(Display* dpy, Window win, struct inter_context* c)
   if(radius < 1)
     radius = 1;
 
-  if (xgwa.width > 2560) scale = 3.5;  /* Retina displays */
+  if (xgwa.width > 2560 || xgwa.height > 2560)
+    scale = 3.5;  /* Retina displays */
   radius *= scale;
 
   create_image(dpy, c, &xgwa);
@@ -852,8 +853,8 @@ static void inter_init(Display* dpy, Window win, struct inter_context* c)
   check_no_mem(dpy, c, c->source);
 
   for(i = 0; i < c->count; i++) {
-    c->source[i].x_theta = frand(2.0)*3.14159;
-    c->source[i].y_theta = frand(2.0)*3.14159;
+    c->source[i].x_theta = frand(2.0)*M_PI;
+    c->source[i].y_theta = frand(2.0)*M_PI;
   }
 
   c->last_frame = float_time();
@@ -890,11 +891,11 @@ static unsigned long do_inter(struct inter_context* c)
 
   for(i = 0; i < c->count; i++) {
     c->source[i].x_theta += (elapsed*c->speed/1000.0);
-    if(c->source[i].x_theta > 2.0*3.14159)
-      c->source[i].x_theta -= 2.0*3.14159;
+    if(c->source[i].x_theta > 2.0*M_PI)
+      c->source[i].x_theta -= 2.0*M_PI;
     c->source[i].y_theta += (elapsed*c->speed/1000.0);
-    if(c->source[i].y_theta > 2.0*3.14159)
-      c->source[i].y_theta -= 2.0*3.14159;
+    if(c->source[i].y_theta > 2.0*M_PI)
+      c->source[i].y_theta -= 2.0*M_PI;
     c->source[i].x = source_x(c, i);
     c->source[i].y = source_y(c, i);
   }
