@@ -7,6 +7,7 @@ SRCS=$*
 TMP1=m6502.h.1.$$
 TMP2=m6502.h.2.$$
 rm -f $TMP1 $TMP2
+trap "rm -f $TMP1 $TMP2" 1 2 3 15 EXIT
 
 if [ -z "$UTILS_SRC" ]; then UTILS_SRC="../utils"; fi
 
@@ -16,9 +17,9 @@ for f in $SRCS ; do
     sed 's/",$/\\n"/' >> $TMP2
   echo ',' >> $TMP2
 done
-rm $TMP1
+rm -f $TMP1
 if cmp -s $TMP2 $TARGET ; then
-  rm $TMP2
+  rm -f $TMP2
 else
   mv $TMP2 $TARGET
 fi
