@@ -93,8 +93,10 @@ struct state {
   Display *dpy;
   Window window;
 
-   XFontStruct *font, *scoreFont;
-   GC draw_gc, erase_gc, level_gc;
+   XftFont *font, *scoreFont;
+   XftColor xft_fg, xft_level_fg;
+   XftDraw *xftdraw;
+   GC draw_gc, erase_gc;
    unsigned int default_fg_pixel;
    XColor scoreColor;
 
@@ -124,6 +126,7 @@ struct state {
 
    int draw_xlim, draw_ylim;
    int draw_reset;
+   int pscale;
 };
 
 
@@ -366,10 +369,11 @@ penetrate_init (Display *dpy, Window window)
 {
   struct state *st = (struct state *) calloc (1, sizeof(*st));
   int i;
-  const char *levelfont = "-*-courier-*-r-*-*-*-380-*-*-*-*-*-*";
-  const char *scorefont = "-*-helvetica-*-r-*-*-*-180-*-*-*-*-*-*";
+  const char *levelfont = "monospace 38";
+  const char *scorefont = "sans-serif 18";
   XGCValues gcv;
   XWindowAttributes xgwa;
+  char *s;
 
   st->dpy = dpy;
   st->window = window;
@@ -377,6 +381,10 @@ penetrate_init (Display *dpy, Window window)
   XGetWindowAttributes (st->dpy, st->window, &xgwa);
   st->cmap = xgwa.colormap;
 
+  st->pscale = 1;
+  if (xgwa.width > 2560 || xgwa.height > 2560)
+    st->pscale *= 3;  /* Retina displays */
+
   st->lrate = 80;
   st->nextBonus = kFirstBonus;
   st->aim = 180;
@@ -388,10 +396,12 @@ penetrate_init (Display *dpy, Window window)
   if (st->lrate < 0) st->lrate = 2;
   st->startlrate = st->lrate;
 
-  st->font = load_font_retry(st->dpy, levelfont);
+  st->font = load_xft_font_retry(st->dpy,  screen_number (xgwa.screen),
+                                 levelfont);
   if (!st->font) abort();
 
-  st->scoreFont = load_font_retry(st->dpy, scorefont);
+  st->scoreFont = load_xft_font_retry(st->dpy, screen_number (xgwa.screen),
+                                      scorefont);
   if (!st->scoreFont) abort();
 
   for (i = 0; i < kMaxMissiles; i++)
@@ -417,13 +427,19 @@ penetrate_init (Display *dpy, Window window)
 
   gcv.foreground = st->default_fg_pixel =
     get_pixel_resource(st->dpy, st->cmap, "foreground", "Foreground");
-  gcv.font = st->scoreFont->fid;
-  st->draw_gc = XCreateGC(st->dpy, st->window, GCForeground | GCFont, &gcv);
-  gcv.font = st->font->fid;
-  st->level_gc = XCreateGC(st->dpy, st->window, GCForeground | GCFont, &gcv);
-  XSetForeground (st->dpy, st->level_gc, st->city[0].color.pixel);
+  st->draw_gc = XCreateGC(st->dpy, st->window, GCForeground, &gcv);
   gcv.foreground = get_pixel_resource(st->dpy, st->cmap, "background", "Background");
   st->erase_gc = XCreateGC(st->dpy, st->window, GCForeground, &gcv);
+  s = get_string_resource (st->dpy, "foreground", "Foreground");
+  if (!s) s = strdup ("white");
+  XftColorAllocName (st->dpy, xgwa.visual, xgwa.colormap, s, &st->xft_fg);
+  free (s);
+
+  /* Level color was st->city[0].color which is hardcoded as: */
+  s = "#FF8811";
+  XftColorAllocName (st->dpy, xgwa.visual, xgwa.colormap, s,
+                     &st->xft_level_fg);
+  st->xftdraw = XftDrawCreate (st->dpy, window, xgwa.visual, xgwa.colormap);
 
 # ifdef HAVE_JWXYZ
   jwxyz_XSetAntiAliasing (st->dpy, st->erase_gc, False);
@@ -449,21 +465,28 @@ static void DrawScore(struct state *st, int xlim, int ylim)
 {
   char buf[16];
   int width, height;
+  XGlyphInfo overall;
   sprintf(buf, "%ld", st->score);
-  width = XTextWidth(st->scoreFont, buf, strlen(buf));
+  XftTextExtentsUtf8 (st->dpy, st->scoreFont, (FcChar8 *) buf, 
+                      strlen(buf), &overall);
+  width = overall.xOff;
   height = font_height(st->scoreFont);
   XSetForeground (st->dpy, st->draw_gc, st->scoreColor.pixel);
   XFillRectangle(st->dpy, st->window, st->erase_gc,
-				  xlim - width - 6, ylim - height - 2, width + 6, height + 2);
-  XDrawString(st->dpy, st->window, st->draw_gc, xlim - width - 2, ylim - 2,
-		    buf, strlen(buf));
+                 xlim - width - 6, ylim - height - 2, width + 6, height + 2);
+  XftDrawStringUtf8 (st->xftdraw, &st->xft_fg, st->scoreFont,
+                     xlim - width - 2, ylim - 2,
+                     (FcChar8 *) buf, strlen(buf));
 
   sprintf(buf, "%ld", st->highscore);
-  width = XTextWidth(st->scoreFont, buf, strlen(buf));
+  XftTextExtentsUtf8 (st->dpy, st->scoreFont, (FcChar8 *) buf, 
+                      strlen(buf), &overall);
+  width = overall.xOff;
   XFillRectangle(st->dpy, st->window, st->erase_gc,
 				  4, ylim - height - 2, width + 4, height + 2);
-  XDrawString(st->dpy, st->window, st->draw_gc, 4, ylim - 2,
-		    buf, strlen(buf));
+  XftDrawStringUtf8 (st->xftdraw, &st->xft_fg, st->scoreFont,
+                     4, ylim - 2,
+                     (FcChar8 *) buf, strlen(buf));
 }
 
 static void AddScore(struct state *st, int xlim, int ylim, long dif)
@@ -484,11 +507,14 @@ static void DrawCity(struct state *st, int x, int y, XColor col)
 {
 	 XSetForeground (st->dpy, st->draw_gc, col.pixel);
 	 XFillRectangle(st->dpy, st->window, st->draw_gc,
-				  x - 30, y - 40, 60, 40);
+                        x - 30 * st->pscale, y - 40 * st->pscale,
+                        60 * st->pscale, 40 * st->pscale);
 	 XFillRectangle(st->dpy, st->window, st->draw_gc,
-						 x - 20, y - 50, 10, 10);
+                        x - 20 * st->pscale, y - 50 * st->pscale, 
+                        10 * st->pscale, 10 * st->pscale);
 	 XFillRectangle(st->dpy, st->window, st->draw_gc,
-				  x + 10, y - 50, 10, 10);
+                        x + 10 * st->pscale, y - 50 * st->pscale,
+                        10 * st->pscale, 10 * st->pscale);
 }
 
 static void DrawCities(struct state *st, int xlim, int ylim)
@@ -521,7 +547,7 @@ static void LoopMissiles(struct state *st, int xlim, int ylim)
 
       /* erase old one */
 
-	 XSetLineAttributes(st->dpy, st->draw_gc, 4, 0,0,0);
+	 XSetLineAttributes(st->dpy, st->draw_gc, 4*st->pscale, 0,0,0);
     XSetForeground (st->dpy, st->draw_gc, m->color.pixel);
 	 XDrawLine(st->dpy, st->window, st->draw_gc,
 				  old_x, old_y, m->x, m->y);
@@ -548,7 +574,7 @@ static void LoopMissiles(struct state *st, int xlim, int ylim)
 		else {
 		  int dx = abs(m->x - b->x);
 		  int dy = abs(m->y - b->y);
-		  int r = b->rad + 2;
+		  int r = b->rad + 2 * st->pscale;
 		  if ((dx < r) && (dy < r))
 			 if (dx * dx + dy * dy < r * r) {
 				m->alive = 0;
@@ -562,7 +588,7 @@ static void LoopMissiles(struct state *st, int xlim, int ylim)
 		float my_pos;
 		/* we just died */
 		Explode(st, m->x, m->y, kBoomRad + max, m->color, 0);
-		XSetLineAttributes(st->dpy, st->erase_gc, 4, 0,0,0);
+		XSetLineAttributes(st->dpy, st->erase_gc, 4*st->pscale, 0,0,0);
 		/* In a perfect world, we could simply erase a line from
 		   (m->startx, m->starty) to (m->x, m->y). This is not a
 		   perfect world. */
@@ -591,7 +617,7 @@ static void LoopLasers(struct state *st, int xlim, int ylim)
 		continue;
 
 	 if (m->oldx != -1) {
-		 XSetLineAttributes(st->dpy, st->erase_gc, 2, 0,0,0);
+		 XSetLineAttributes(st->dpy, st->erase_gc, 2*st->pscale, 0,0,0);
 		 XDrawLine(st->dpy, st->window, st->erase_gc,
 				  m->oldx2, m->oldy2, m->oldx, m->oldy);
 	 }
@@ -607,7 +633,7 @@ static void LoopLasers(struct state *st, int xlim, int ylim)
 	 m->oldx = x;
 	 m->oldy = y;
 
-	 XSetLineAttributes(st->dpy, st->draw_gc, 2, 0,0,0);
+	 XSetLineAttributes(st->dpy, st->draw_gc, 2*st->pscale, 0,0,0);
     XSetForeground (st->dpy, st->draw_gc, m->color.pixel);
 	 XDrawLine(st->dpy, st->window, st->draw_gc,
 				  m->x, m->y, x, y);
@@ -630,7 +656,7 @@ static void LoopLasers(struct state *st, int xlim, int ylim)
 		  else {
 			 int dx = abs(m->x - b->x);
 			 int dy = abs(m->y - b->y);
-			 int r = b->rad + 2;
+			 int r = b->rad + 2 * st->pscale;
 			 if (b->oflaser)
 				continue;
 			 if ((dx < r) && (dy < r))
@@ -665,13 +691,23 @@ static void LoopBooms(struct state *st, int xlim, int ylim)
 		  m->rad++;
 		  if (m->rad >= m->max)
 			 m->outgoing = 0;
-		  XSetLineAttributes(st->dpy, st->draw_gc, 1, 0,0,0);
+		  XSetLineAttributes(st->dpy, st->draw_gc, 1*st->pscale, 0,0,0);
 		  XSetForeground (st->dpy, st->draw_gc, m->color.pixel);
-		  XDrawArc(st->dpy, st->window, st->draw_gc, m->x - m->rad, m->y - m->rad, m->rad * 2, m->rad * 2, 0, 360 * 64);
+		  XDrawArc(st->dpy, st->window, st->draw_gc,
+                           m->x - m->rad * st->pscale,
+                           m->y - m->rad * st->pscale,
+                           m->rad * 2 * st->pscale,
+                           m->rad * 2 * st->pscale,
+                           0, 360 * 64);
 		}
 		else {
-		  XSetLineAttributes(st->dpy, st->erase_gc, 1, 0,0,0);
-		  XDrawArc(st->dpy, st->window, st->erase_gc, m->x - m->rad, m->y - m->rad, m->rad * 2, m->rad * 2, 0, 360 * 64);
+		  XSetLineAttributes(st->dpy, st->erase_gc, 1*st->pscale, 0,0,0);
+		  XDrawArc(st->dpy, st->window, st->erase_gc,
+                           m->x - m->rad * st->pscale,
+                           m->y - m->rad * st->pscale,
+                           m->rad * 2 * st->pscale,
+                           m->rad * 2 * st->pscale,
+                           0, 360 * 64);
 		  m->rad--;
 		  if (m->rad <= 0)
 			 m->alive = 0;
@@ -719,6 +755,7 @@ static void NewLevel(struct state *st, int xlim, int ylim)
   int width, i, sumlive = 0;
   int liv[kNumCities];
   int freecity = 0;
+  XGlyphInfo overall;
 
   if (st->level == 0) {
 	 st->level++;
@@ -753,9 +790,12 @@ static void NewLevel(struct state *st, int xlim, int ylim)
 		sprintf(buf, "GAME OVER");
   }
   if (st->level > 0) {
-	 width = XTextWidth(st->font, buf, strlen(buf));
-	 XDrawString(st->dpy, st->window, st->level_gc, xlim / 2 - width / 2, ylim / 2 - font_height(st->font) / 2,
-					 buf, strlen(buf));
+         XftTextExtentsUtf8 (st->dpy, st->font, (FcChar8 *) buf, 
+                             strlen(buf), &overall);
+         width = overall.xOff;
+         XftDrawStringUtf8 (st->xftdraw, &st->xft_level_fg, st->font,
+                   xlim / 2 - width / 2, ylim / 2 - font_height(st->font) / 2,
+                            (FcChar8 *) buf, strlen(buf));
 	 XSync(st->dpy, False);
 	 usleep(1000000);
   }
@@ -765,17 +805,22 @@ static void NewLevel(struct state *st, int xlim, int ylim)
 		int sumwidth;
 		/* draw live cities */
 		XFillRectangle(st->dpy, st->window, st->erase_gc,
-							0, ylim - 100, xlim, 100);
+                               0, ylim - 100 * st->pscale, xlim, 100 * st->pscale);
 
 		sprintf(buf, "X %ld", st->level * 100L);
 		/* how much they get */
-		sumwidth = XTextWidth(st->font, buf, strlen(buf));
+                XftTextExtentsUtf8 (st->dpy, st->font, (FcChar8 *) buf, 
+                                    strlen(buf), &overall);
+                sumwidth = overall.xOff;
 		/* add width of city */
 		sumwidth += 60;
 		/* add spacer */
 		sumwidth += 40;
 		DrawCity(st, xlim / 2 - sumwidth / 2 + 30, ylim * 0.70, st->city[0].color);
-		XDrawString(st->dpy, st->window, st->level_gc, xlim / 2 - sumwidth / 2 + 40 + 60, ylim * 0.7, buf, strlen(buf));
+                XftDrawStringUtf8 (st->xftdraw, &st->xft_level_fg, st->font,
+                                   xlim / 2 - sumwidth / 2 + 40 + 60,
+                                   ylim * 0.7, 
+                                   (FcChar8 *) buf, strlen(buf));
 		for (i=0;i<kNumCities;i++) {
 		  if (liv[i]) {
 			 st->city[i].alive = 1;
@@ -812,15 +857,19 @@ static void NewLevel(struct state *st, int xlim, int ylim)
 		  if (!--ncnt)
 			 st->city[i].alive = 1;
 	 strcpy(buf, "Bonus City");
-	 width = XTextWidth(st->font, buf, strlen(buf));
-	 XDrawString(st->dpy, st->window, st->level_gc, xlim / 2 - width / 2, ylim / 4, buf, strlen(buf));
+         XftTextExtentsUtf8 (st->dpy, st->font, (FcChar8 *) buf, 
+                             strlen(buf), &overall);
+         width = overall.xOff;
+         XftDrawStringUtf8 (st->xftdraw, &st->xft_level_fg, st->font,
+                            xlim / 2 - width / 2, ylim / 4,
+                            (FcChar8 *) buf, strlen(buf));
 	 DrawCities(st, xlim, ylim);
 	 XSync(st->dpy, False);
 	 usleep(1000000);
   }
 
   XFillRectangle(st->dpy, st->window, st->erase_gc,
-					  0, 0, xlim, ylim - 100);
+                 0, 0, xlim, ylim - 100 * st->pscale);
   
   if (!st->bround)
 	 st->level++;
@@ -841,12 +890,17 @@ static void NewLevel(struct state *st, int xlim, int ylim)
 		for (i=0;i<kNumCities;i++)
 		  st->blive[i] = st->city[i].alive;
 		sprintf(buf, "Bonus Round");
-		width = XTextWidth(st->font, buf, strlen(buf));
-		XDrawString(st->dpy, st->window, st->level_gc, xlim / 2 - width / 2, ylim / 2 - font_height(st->font) / 2, buf, strlen(buf));
+                XftTextExtentsUtf8 (st->dpy, st->font, (FcChar8 *) buf, 
+                                    strlen(buf), &overall);
+                width = overall.xOff;
+                XftDrawStringUtf8 (st->xftdraw, &st->xft_level_fg, st->font,
+                                   xlim / 2 - width / 2,
+                                   ylim / 2 - font_height(st->font) / 2,
+                                   (FcChar8 *) buf, strlen(buf));
 		XSync(st->dpy, False);
 		usleep(1000000);
 		XFillRectangle(st->dpy, st->window, st->erase_gc,
-							0, 0, xlim, ylim - 100);
+                               0, 0, xlim, ylim - 100 * st->pscale);
 	 }
   }
 
@@ -953,15 +1007,15 @@ penetrate_free (Display *dpy, Window window, void *closure)
   struct state *st = (struct state *) closure;
   XFreeGC (dpy, st->draw_gc);
   XFreeGC (dpy, st->erase_gc);
-  XFreeGC (dpy, st->level_gc);
-  XFreeFont (dpy, st->font);
-  XFreeFont (dpy, st->scoreFont);
+  XftFontClose (st->dpy, st->font);
+  XftFontClose (st->dpy, st->scoreFont);
+  XftDrawDestroy (st->xftdraw);
   free (st);
 }
 
 
 static const char *penetrate_defaults [] = {
-  ".lowrez:     true",
+/*  ".lowrez:     true", */
   ".background:	black",
   ".foreground:	white",
   "*fpsTop:	true",
