@@ -55,9 +55,6 @@ static const char sccsid[] = "@(#)polyominoes.c 5.01 2000/12/18 xlockmore";
 static Bool identical;
 static Bool plain;
 
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
-
 static XrmOptionDescRec opts[] =
 {
   {"-identical", ".polyominoes.identical", XrmoptionNoArg, "on"},
@@ -975,6 +972,10 @@ static void create_bitmaps(ModeInfo * mi, polyominoesstruct *sp)
   }
 
   sp->use_bitmaps = 1;
+
+#ifdef HAVE_ANDROID  /* Doesn't work -- displays black */
+  sp->use_bitmaps = 0;
+#endif
 }
 
 static void draw_with_bitmaps(ModeInfo * mi, polyominoesstruct *sp)
